import re

from django import template
from django.utils.encoding import smart_str
from sorl.thumbnail import default
from django.template import TemplateSyntaxError

from sorl.thumbnail.templatetags.thumbnail import ThumbnailNode
from .thumbnail import settings
from .utils import get_thumbnail_options_from_dict

register = template.Library()
kw_pat = re.compile(r'^(?P<key>[\w]+)=(?P<value>.+)$')

class CustomThumbnailNode(ThumbnailNode):
    """ Extends ThumbnailNode to use thumbnail sizes from settings"""
    error_message = ('Please enter sizes defined in settings')
    def __init__(self, parser, token):
        ThumbnailNode.__init__(self, parser, token)
        bits = token.split_contents()
        if len(bits) < 5 or bits[-2] != 'as':
            raise TemplateSyntaxError(self.error_msg)
        try:
            THUMBNAIL_OPTIONS = get_thumbnail_options_from_dict(settings.THUMBNAIL_OPTIONS_DICT)
            thumbnail_options = settings.THUMBNAIL_OPTIONS[bits[2]]
            thumbnail_options = thumbnail_options.split()
        except KeyError:
            raise TemplateSyntaxError(self.error_message)
        self.geometry = parser.compile_filter("'%s'" % thumbnail_options[0])
        self.options = []
        try:
            m = kw_pat.match(thumbnail_options[1])
            key = smart_str(m.group('key'))
            expr = parser.compile_filter('"%s"' % m.group('value'))
            self.options.append((key, expr))
        except IndexError:
            pass
        self.as_var = bits[-1]

@register.tag
def thumbnail(parser, token):
    return CustomThumbnailNode(parser, token)


from sorl.thumbnail.base import ThumbnailBackend
from sorl.thumbnail import default
from sorl.thumbnail.images import ImageFile, DummyImageFile

class AsyncThumbnailBackend(ThumbnailBackend):
    def get_thumbnail(self, file_, geometry_string, **options):
        source = ImageFile(file_)
        for key, value in self.default_options.iteritems():
            options.setdefault(key, value)
        name = self._get_thumbnail_filename(source, geometry_string, options)
        thumbnail = ImageFile(name, default.storage)
        cached = default.kvstore.get(thumbnail)
        if cached:
            return cached
        # We don't check if thumbnail exists as sorl-thumbnail does. It becomes
        # very costly for remote storages.
        # Furthermore, I have added following code to reduce/prevent duplicate
        # tasks in celery. It's hacky.
        try:
            default.kvstore.get_or_set(source)
            default.kvstore.set(thumbnail, source)
            return thumbnail
        except IOError:
            pass
        # Finally, if there is no thumbnail, we create one.
        from .tasks import create_thumbnail
        job = create_thumbnail.delay(file_, geometry_string, **options)
        # Sometimes thumbnail generation takes quite some time, show dummy.
        if job:
            return DummyImageFile(geometry_string)


# Template tag doesn't work with options dictionary. We need to change it.
# Pass dictionary from setting file.

def get_thumbnail_options_from_dict(dictt):
    ret= {}
    for name, options in dictt.iteritems():
        opt = copy(options)
        geometry = opt.pop('geometry')
        new_options = ['%s=%s' % (k, v) for k, v in opt.iteritems()]
        new_options.insert(0, geometry)
        ret[name] = ' '.join(new_options)
    return ret


from django.conf import settings

# Default sorl-thumbnail backend is overridden. We need to tell which backend to use.
THUMBNAIL_BACKEND = 'sorl-thumbnail-async.thumbnail.backend.AsyncThumbnailBackend'

# Instead of original sorl-thumbnail design, we predefine the thumbnail options
# here and reuse same everywhere. 
THUMBNAIL_OPTIONS_DICT = {
        'small': {
                'geometry': '140x140',
                'crop': 'center'
        }
    }


import celery

from sorl.thumbnail import default
from sorl.thumbnail.images import ImageFile

@celery.task
def create_thumbnail(image_file, geometry_string, **options):
    # Note that thumbnail options must be same for a type of thumbnail.
    # Otherwise, different thumbnails are created.
    source = ImageFile(image_file)
    for key, value in default.backend.default_options.iteritems():
            options.setdefault(key, value)
    name = default.backend._get_thumbnail_filename(source, geometry_string, options)
    thumbnail = ImageFile(name, default.storage)
    source_image = default.engine.get_image(source)
    default.backend._create_thumbnail(source_image, geometry_string, options, thumbnail)

    # Need to set size to store in kvstore.
    size = default.engine.get_image_size(source_image)
    source.set_size(size)

    default.kvstore.get_or_set(source)
    default.kvstore.set(thumbnail, source)


from copy import copy
from django.db.models import signals
from django.dispatch import receiver

from .thumbnail import settings
from .tasks import create_thumbnail

class AsyncThumbnailMixin(object):
    """
    All model which have ImageField to be thumbnailed inheret from this class. 
    """
    image_field_name = 'picture'

    def call_upload_task(self):
        for name, options in settings.THUMBNAIL_OPTIONS_DICT.iteritems():
            opt = copy(options)
            geometry = opt.pop('geometry')
            create_thumbnail.delay(getattr(self, self.image_field_name), geometry, **opt)

    def save(self, *args, **kwargs):
        super(AsyncThumbnailMixin, self).save(*args, **kwargs)
        self.call_upload_task()



from setuptools import setup, find_packages

version = '0.4.4'

setup(
    name='sorl-thumbnail-async',
    version=version,
    description='Asynchronous thumbnailing app in django with remote storages like S3',
    author='Nar Kumar Chhantyal',
    author_email='neokya@gmail.com',
    url='https://github.com/neokya/sorl-thumbnail-async',
    packages=find_packages(),
    zip_safe=False,
    include_package_data=True,
    install_requires=[
        'Django>=1.3',
    ],
)